"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertStyledPropToCSS = void 0;
const css_keys_1 = __importDefault(require("../styled-props/css-keys"));
const object_1 = require("./object");
const breakpoints = ['', 'Sm', 'Md', 'Lg', 'Xl'];
const breakpointMap = {
    Sm: 'small',
    Md: 'medium',
    Lg: 'large',
    Xl: 'xlarge',
};
/**
 * Converts a SRC Styled prop to CSS property for it to get written in the Styled component CSS.
 * If value is CSS selector, it converts value's children to CSS properties and wraps them in the selector.
 *
 * If key has 'get' as prefix and its value is function, treats it as a getter function and fetches the CSS by calling the function for each given breakpoint.
 *
 * @param key - Styled prop name
 * @param props - Given Styled props
 * @param theme
 */
const convertStyledPropToCSS = (key, props, theme) => {
    var _a;
    const { cssInput, breakpoint } = extractBreakpointFromKey(key);
    let value = props[key];
    if (!cssInput) {
        if (key.startsWith('get') && typeof value === 'function') {
            let result = '';
            let propKey = key.replace(/^get/, '');
            propKey = propKey.substring(0, 1).toLowerCase() + propKey.substring(1);
            breakpoints.forEach(breakpointKey => {
                const prop = props[`${propKey}${breakpointKey}`];
                if (typeof prop !== 'undefined') {
                    result += wrapCSSInMediaRule(value(prop, theme, breakpointKey, props), breakpointMap[breakpointKey], theme);
                }
            });
            return result;
        }
        return null;
    }
    let important = false;
    if (Array.isArray(value)) {
        [value, important] = value;
    }
    if (value === null) {
        return null;
    }
    let css = '';
    if ((0, object_1.isObject)(value) && cssInput.startsWith('&')) {
        css = cssInput.replace('|', Object.keys(value).map(i => (0, exports.convertStyledPropToCSS)(i, value, theme)).join(' '));
    }
    else if (!(0, object_1.isObject)(value) && !cssInput.startsWith('&')) {
        if (typeof value === 'number') {
            if (value !== 0) {
                value = `${value}px`;
            }
        }
        else if (typeof value === 'string') {
            (_a = value.match(/((\$|!)[\S]+)|((\{|\[)[\S]+(\}|\]))/g)) === null || _a === void 0 ? void 0 : _a.forEach(str => {
                if (str.startsWith('$')
                    || (str.startsWith('{') && str.endsWith('}'))) {
                    const themeAttr = str.replace(/^\$(.*?)$/, '$1').replace(/\{(.*?)\}/, '$1');
                    const themeAttrVal = themeAttr.split('.').reduce((a, b) => a === null || a === void 0 ? void 0 : a[b], theme.$);
                    if (typeof themeAttrVal !== 'undefined') {
                        value = value.replace(str, themeAttrVal);
                    }
                }
                else if (str.startsWith('!')
                    || (str.startsWith('[') && str.endsWith(']'))) {
                    const paletteAttr = str.replace(/^!(.*?)$/, '$1').replace(/\[(.*?)\]/, '$1');
                    let paletteAttrVal = paletteAttr.split('.').reduce((a, b) => a === null || a === void 0 ? void 0 : a[b], theme.$.palette);
                    if (typeof paletteAttrVal === 'object') {
                        paletteAttrVal = paletteAttrVal.main;
                    }
                    if (typeof paletteAttrVal !== 'undefined') {
                        value = value.replace(str, paletteAttrVal);
                    }
                }
            });
        }
        else {
            return null;
        }
        if (important) {
            value = `${value} !important`;
        }
        css = cssInput.split('|').map(i => `${i}:${value};`).join(' ');
    }
    else {
        return null;
    }
    return wrapCSSInMediaRule(css, breakpoint, theme);
};
exports.convertStyledPropToCSS = convertStyledPropToCSS;
/**
 * Extracts breakpoint key out of the Styled prop name.
 *
 * @param key - Styled prop name
 * @returns - cssInput is CSS property or selector from cssKeys.
 */
const extractBreakpointFromKey = (key) => {
    let cssInput = css_keys_1.default[key];
    if (cssInput) {
        return {
            cssInput: css_keys_1.default[key],
            breakpoint: null,
        };
    }
    cssInput = css_keys_1.default[key.substring(0, key.length - 2)];
    const breakpoint = breakpointMap[key.substring(key.length - 2)];
    return {
        cssInput,
        breakpoint,
    };
};
/**
 * Wraps CSS in the media rule.
 *
 * @param css
 * @param breakpoint
 * @param theme
 */
const wrapCSSInMediaRule = (css, breakpoint, theme) => {
    const width = theme.$.length.breakpoint[breakpoint];
    return typeof width !== 'undefined'
        ? `@media(min-width:${width}px) { ${css} }`
        : css;
};
